﻿using System;
using System.Collections.Generic;
using System.Linq;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Shapes;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Documents;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;

// The Templated Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234235


namespace SimpleSliderDemo.Controls
{
    [TemplatePart(Name = ThumbPartName, Type = typeof(Thumb))]
    [TemplatePart(Name = TrackPartName, Type = typeof(Rectangle))]
    public sealed class SimpleSlider : Control
    {

        #region Fields

        private const string ThumbPartName = "PART_Thumb";
        private const string TrackPartName = "PART_Track";

        private Thumb thumb;
        private Rectangle rectangle;

        #endregion //Fields

        #region Constructor
        public SimpleSlider()
        {
            this.DefaultStyleKey = typeof(SimpleSlider);
        }
        #endregion //Constructor

        #region Properties

        #region MinimumValue

        // Using a DependencyProperty as the backing store for MinimumValue.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MinimumValueProperty =
            DependencyProperty.Register("MinimumValue", typeof(object), typeof(SimpleSlider), new PropertyMetadata(0.0));

        /// <summary>
        /// Gets or sets the minimum.
        /// </summary>
        public double MinimumValue
        {
            get { return (double)GetValue(MinimumValueProperty); }
            set { SetValue(MinimumValueProperty, value); }
        }

        #endregion //MinimumValue

        #region MaximumValue

        // Using a DependencyProperty as the backing store for MaximumValue.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MaximumValueProperty =
            DependencyProperty.Register("MaximumValue", typeof(object), typeof(SimpleSlider), new PropertyMetadata(0.0));

        /// <summary>
        /// Gets or sets the maximum.
        /// </summary>
        public double MaximumValue
        {
            get { return (double)GetValue(MaximumValueProperty); }
            set { SetValue(MaximumValueProperty, value); }
        }

        #endregion //MaximumValue

        #region Value

        // Using a DependencyProperty as the backing store for Value.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ValueProperty =
            DependencyProperty.Register("Value", typeof(object), typeof(SimpleSlider), new PropertyMetadata(0.0, OnValueChanged));

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        public double Value
        {
            get { return (double)GetValue(ValueProperty); }
            set
            {
                SetValue(ValueProperty, value);
            }
        }
        #endregion //Value

        #endregion //Properties

        #region Methods

        #region OnApplyTemplate
        /// <summary>
        /// When overridden in a derived class, is invoked whenever application code or internal processes call <see cref="M:System.Windows.FrameworkElement.ApplyTemplate"/>.
        /// </summary>
        protected override void OnApplyTemplate() // OnApplyTemplateCore()
        {
            //base.OnApplyTemplateCore();

            base.OnApplyTemplate();

            this.thumb = this.GetTemplateChild(ThumbPartName) as Thumb;
            if (this.thumb != null)
            {
                this.thumb.DragDelta += this.Thumb_DragDelta;
            }

            this.rectangle = this.GetTemplateChild(TrackPartName) as Rectangle;

            this.SizeChanged += new SizeChangedEventHandler(SimpleSlider_SizeChanged);
        }
        #endregion //OnApplyTemplate

        #region SimpleSlider_SizeChanged
        /// <summary>
        /// Called when size changed.
        /// </summary>
        private void SimpleSlider_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (e.NewSize.Width != e.PreviousSize.Width)
            {
                this.UpdateControlParts();
            }
        }
        #endregion //SimpleSlider_SizeChanged

        #region OnValueChanged
        /// <summary>
        /// Called when value changed.
        /// </summary>
        private static void OnValueChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            var customSlider = (SimpleSlider)dependencyObject;
            customSlider.UpdateControlParts();
        }
        #endregion //OnValueChanged

        #region Thumb_DragDelta
        /// <summary>
        /// Handles the DragDelta event of the Thumb control.
        /// </summary>
        private void Thumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            var pixelDiff = e.HorizontalChange;
            var currentLeft = Canvas.GetLeft(this.thumb);

            // trying to drag too far left
            if ((currentLeft + pixelDiff) < 0)
            {
                this.Value = 0;
            }
            // trying to drag too far right
            else if ((currentLeft + pixelDiff + this.thumb.ActualWidth) > this.ActualWidth)
            {
                this.Value = this.MaximumValue;
            }
            else
            {
                var totalSize = this.ActualWidth;
                var ratioDiff = pixelDiff / totalSize;
                var rangeSize = this.MaximumValue - this.MinimumValue;
                var rangeDiff = rangeSize * ratioDiff;
                this.Value += rangeDiff;
            }
        }
        #endregion //Thumb_DragDelta

        #region UpdateControlParts
        /// <summary>
        /// Updates the control parts.
        /// </summary>
        private void UpdateControlParts()
        {
            double halfTheThumbWith = 0;

            if (this.thumb != null)
            {
                halfTheThumbWith = this.thumb.ActualWidth / 2;
            }

            double totalSize = this.ActualWidth - halfTheThumbWith * 2;

            double ratio = totalSize / (this.MaximumValue - this.MinimumValue);

            if (this.thumb != null)
            {
                Canvas.SetLeft(this.thumb, ratio * this.Value);
            }

            if (this.rectangle != null)
            {
                this.rectangle.Width = ratio * this.Value + halfTheThumbWith;
            }
        }
        #endregion //UpdateControlParts

        #endregion //Methods
    }
}
